function TableOfVarsAddForm(
    table       // TableOfVars instance
) {
    this.table = table;
    const doc = table.node.ownerDocument;
    const tbody = doc.createElement('tbody');
    const tr = doc.createElement('tr');
    tr.innerHTML = '<td><input /></td><td><select></select></td><td><button title="Variable ergänzen">➕ Variable</button></td>';
    this._inputName = tr.getElementsByTagName('input')[0];
    this._inputType = tr.getElementsByTagName('select')[0];
    const optgroup = doc.createElement('optgroup');
    optgroup.setAttribute('label', 'Grundlegend');
    this._inputType.appendChild(optgroup);
    tr.getElementsByTagName('button')[0].addEventListener('click', () => this.submit());
    TableOfVars.varTypes.slice().sort().forEach(({ typeName }) => {
        const option = doc.createElement('option');
        option.textContent = typeName;
        optgroup.appendChild(option);
        if (typeName == this._lastSelectedType)
            option.selected = true;
    });
    this._livenTypeSelect(this._inputType);
    tbody.appendChild(tr);
    table.node.appendChild(tbody);
    this.node = tbody;
    this.typeSelects = [ this._inputType ];
    this.reloadBaseType();
}

/** Used tbody element */
TableOfVarsAddForm.prototype.node = null;

/** optgroup element for class names */
TableOfVarsAddForm.prototype._optgroupClasses = null;

/** Used inputs to select type */
TableOfVarsAddForm.prototype.typeSelects = [];

/** Available class names */
Object.defineProperty(TableOfVarsAddForm.prototype, 'classNames', {
    enumerable: true,
    get() {
        if (!this._optgroupClasses)
            return [];
        return Array.from(this._optgroupClasses.getElementsByTagName('option')).map(node => node.textContent);
    },
    set(arrNames) {
        if (!(arrNames instanceof Array) || !arrNames.length) {
            if (this._optgroupClasses) {
                this._optgroupClasses.parentNode.removeChild(this._optgroupClasses);
                this._optgroupClasses = null;
            }
            return;
        }
        const doc = this.node.ownerDocument;
        const optgroup = doc.createElement('optgroup');
        optgroup.setAttribute('label', 'Klassen');
        arrNames.forEach(name => ScoTools.createHtml('option', optgroup, null, name));
        if (this._optgroupClasses)
            this._optgroupClasses.parentNode.replaceChild(optgroup, this._optgroupClasses);
        else
            this._inputType.insertBefore(optgroup, this._inputType.getElementsByTagName('optgroup')[0].nextSibling);
        this._optgroupClasses = optgroup;
    }
});

/**
* Use inputed values to create a new variable
* @return undefined
*/
TableOfVarsAddForm.prototype.submit = function() {
    this.table.addVar(
        this._inputName.value.trim(),
        this._lastSelectedType,
        undefined,
        this.typeSelects.length - 1
    );
}

/** Label for array option */
TableOfVarsAddForm.prototype.labelArray = 'Array aus';

/** Can arrays be created? */
TableOfVarsAddForm.prototype._allowArrays = false;
Object.defineProperty(TableOfVarsAddForm.prototype, 'allowArrays', {
    enumerable: true,
    get() { return this._allowArrays; },
    set(v) {
        if (v == this._allowArrays)
            return;
        if (v) {
            ScoTools.createHtml('option', this.typeSelects[0], null, this.labelArray, this.typeSelects[0].firstChild);
            this._allowArrays = true;
        } else {
            const remainingInput = this.typeSelects[0];
            remainingInput.removeChild(remainingInput.firstChild);
            this.typeSelects.slice(1, this.typeSelects.length).forEach(node => node.parentNode.removeChild(node));
            this.typeSelects = [ remainingInput];
            this.reloadBaseType();
            this._allowArrays = false;
        }
    }
});

/** Recently selected baseType */
TableOfVarsAddForm.prototype._lastSelectedType = 'int';

/**
* Reset selected base type to last selected base type
* @return undefined
*/
TableOfVarsAddForm.prototype.reloadBaseType = function() {
    const select = this.typeSelects[this.typeSelects.length - 1];
    select.selectedIndex = Math.max(Array.from(select.options).map(option => option.text).indexOf(this._lastSelectedType), this.allowArrays ? 1 : 0);
}

/**
* Add event listener to type select input `node`
* @return undefined
*/
TableOfVarsAddForm.prototype._livenTypeSelect = function(
    node        // HTMLSelectionElement to add event listener to
) {
    node.addEventListener('change', () => {
        const index = this.typeSelects.indexOf(node);
        if (index < 0)
            return;
        const arraySelected = (!node.selectedIndex && this.allowArrays);
        if (arraySelected) {
            if (index + 1 < this.typeSelects.length)
                return;
            const clone = node.cloneNode(true);
            this._livenTypeSelect(clone);
            node.parentNode.appendChild(clone);
            this.typeSelects.push(clone);
            this.reloadBaseType();
        } else {
            this._lastSelectedType = node.options[node.selectedIndex].text;
            while (index + 1 < this.typeSelects.length) {
                const select = this.typeSelects.pop();
                select.parentNode.removeChild(select);
            }
        }
    });
}
